%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [envMapGround, envMapGroundMask] = addGroundToEnvMap(img, skyMask, horizonLine)
%  Extracts an environment map from a single image by taking a circle and projecting it onto a
%  sphere. (from Khan, Reinhard, et al. paper "Image-based material editing", SIGGRAPH 2006)
%
% Input parameters:
%
% Output parameters:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [envMapGround, envMapGroundMask] = addGroundToEnvMap(img, skyMask, sceneMask, horizonLine, focalLength, camZenith, envMapHeight)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2009-2011 Jean-Francois Lalonde
% Carnegie Mellon University
% Do not distribute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

horizonRowInd = round(size(img,1)/2-horizonLine);

% crop image at bottom if scene doesn't go all the way down
[rScene, cScene] = find(sceneMask);
img = img(1:max(rScene), :, :);
skyMask = skyMask(1:max(rScene),:);

% squish (or expand) what's below the horizon to make it align with 0.5 along y 
imgDest = squishImage(img, horizonRowInd, 250);
skyMaskDest = squishImage(skyMask, horizonRowInd, 250);

% project onto a sphere
envMapGround = imageCircle2EnvMap(imgDest, 'EnvMapHeight', envMapHeight);
envMapGroundMask = imageCircle2EnvMap(repmat(~skyMaskDest, [1 1 3]), 'EnvMapHeight', envMapHeight);  

% find angle of tallest point in the scene.
[rImg,c] = find(~skyMask);
maxAngleImg = atan2(size(skyMask,1)/2-min(rImg), focalLength) + (pi/2-camZenith);

% make sure to align the environemnt map to that highest point.
[rEnvMap,c] = find(envMapGroundMask(:,:,1));
maxPos = (size(envMapGroundMask,2)-min(rEnvMap))/size(envMapGroundMask,2);
maxAngleEnvMap = sin(maxPos);

envMapGroundMask = warpToSphereSlice(double(envMapGroundMask(:,:,1)), maxAngleImg, maxAngleEnvMap, min(rEnvMap));
envMapGround = warpToSphereSlice(envMapGround, maxAngleImg, maxAngleEnvMap, min(rEnvMap));

%% Useful function
function warpedEnvMap = warpToSphereSlice(envMap, maxAngleImg, maxAngleEnvMap, rEnvMap)

% warp env map such that the maximum point aligns with the maximum angle
rowsToSwap = rEnvMap:size(envMap,1)/2;
nbCols = size(envMap,2);

if length(rowsToSwap) <= 1
    % don't warp it
    warpedEnvMap = envMap;
    return;
end

% warp upper half only
warpedEnvMap = zeros(length(rowsToSwap), size(envMap, 2));
for c=1:size(envMap,3)
    warpedEnvMap(:,:,c) = interp2(1:nbCols, sin(linspace(maxAngleImg, 0, length(rowsToSwap))), envMap(rowsToSwap,:,c), 1:nbCols, sin(linspace(maxAngleEnvMap,0,length(rowsToSwap)))');
end
warpedEnvMap = cat(1, zeros(min(rowsToSwap)-1, size(envMap,2), size(envMap, 3)), warpedEnvMap, envMap(size(envMap,1)/2+1:end,:,:));
warpedEnvMap(isnan(warpedEnvMap)) = 0;


%% Useful function
function squishedImg = squishImage(img, horizonRowInd, height)

imgGround = img(horizonRowInd:end, :, :);
imgSky = img(1:horizonRowInd-1,:,:);

squishedImg = zeros(height.*2, size(img, 2), size(img,3));

squishedImg(1:height,:,:) = imresize(imgSky, [height, size(img,2)]);
squishedImg(height+1:end,:,:) = imresize(imgGround, [height, size(img,2)]);
