%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [outputAngularEnvMap, sunLightStr, sunColor, sunBrightnessRatio] = addSunToEnvMap(angularEnvMap, sunZenith, sunAzimuth, ...
%    sunModelMean, sunModelDirs, airMassModelParams, sunOcclusionFactor, varargin)
%  Adds a sun to an existing environment map (in angular
%  format). Can eiher add it directly, or as an external light
%  source. Uses the PCA sun database from P. Debevec's sky light
%  probes.
%
% Input parameters:
%
% Output parameters:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [outputAngularEnvMap, sunLightStr, sunColor, sunBrightnessRatio] = addSunToEnvMap(angularEnvMap, sunZenith, sunAzimuth, ...
    sunModelMean, sunModelDirs, airMassModelParams, sunOcclusionFactor, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2006-2011 Jean-Francois Lalonde
% Carnegie Mellon University
% Do not distribute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
% addpath ../;
% setPath;

% Parse inputs
defaultArgs = struct('DoDisplay', 0, 'DoSunLight', 0);
args = parseargs(defaultArgs, varargin{:}); 

maxSunZenithBrightness = 0.8e5; 
minSunOcclusionFactor = 0.01;

sunOcclusionRatio = (1-sunOcclusionFactor).*(1-minSunOcclusionFactor)+minSunOcclusionFactor;
opticalMassRatio = attOpticalMass(airMassModelParams(1,:), airMassModelParams(2,:), airMassModelParams(3,:), sunZenith) ./ attOpticalMass(airMassModelParams(1,:), airMassModelParams(2,:), airMassModelParams(3,:), 0);
sunBrightnessRatio = sunOcclusionRatio .* opticalMassRatio;

sunBrightness = maxSunZenithBrightness.*sunBrightnessRatio;

% take the mean, warp it, normalize it by using our parametric model
sunPatch = exp(sunModelMean);

% fprintf('max sun patch (before warping): %.2f\n', max(sunPatch(:)));

%% 
if args.DoSunLight
    % no change to the environment map
    outputAngularEnvMap = angularEnvMap;
    
    % direction
    sunX = sin(sunZenith).*sin(sunAzimuth);
    sunY = cos(sunZenith);
    sunZ = -sin(sunZenith).*cos(sunAzimuth);
    
    % solid angle
    minAngle = 0.5;
    maxAngle = 3;
    
    % don't modify the angle
    sunAngle = minAngle;%minAngle + (1-sunOcclusionFactor).*(maxAngle-minAngle);
    
    sunColor = sunBrightness;
    
    fprintf('sun color: (%.2f, %.2f, %.2f)...', sunColor(1), sunColor(2), sunColor(3));

    % write the string
    sunLightStr = '# Automatically generated sun light\n';
    sunLightStr = cat(2, sunLightStr, 'void light solar\n0\n0\n');
    % color (HDR!)
    sunLightStr = cat(2, sunLightStr, sprintf('3 %f %f %f\n\n', sunColor(1), sunColor(2), sunColor(3))); 
    
    sunLightStr = cat(2, sunLightStr, 'solar source sun\n0\n0\n');
    % direction + angle
    sunLightStr = cat(2, sunLightStr, sprintf('4 %f %f %f %f\n', sunX, sunY, sunZ, sunAngle)); 
else
    sunLightStr = '';
    
    %% Get 3-D directions of the existing environment map
    [dx,dy,dz] = envmapAngular2World(size(angularEnvMap,1));
    dEnvMap = [dx(:) dy(:) dz(:)]';

    rEnvMap = acos(-dEnvMap(3,:)) ./ (2.*pi.*sqrt(dEnvMap(1,:).^2 + dEnvMap(2,:).^2));
    uEnvMap = 1/2-rEnvMap.*dEnvMap(2,:);
    vEnvMap = 1/2+rEnvMap.*dEnvMap(1,:);

    %% Center the sun patch at its position
    % azimuth: about y axis
    R1 = [cos(-sunAzimuth) 0 sin(-sunAzimuth); 0 1 0; -sin(-sunAzimuth) 0 cos(-sunAzimuth)];

    % zenith: about x axis
    R2 = [1 0 0; 0 cos(pi/2-sunZenith) -sin(pi/2-sunZenith); 0 sin(pi/2-sunZenith) cos(pi/2-sunZenith)];

    % compose rotations
    R = R1*R2;
    dSunRot = R*sunModelDirs;

    rSunRot = acos(-dSunRot(3,:)) ./ (2.*pi.*sqrt(dSunRot(1,:).^2 + dSunRot(2,:).^2));
    uSunRot = 1/2-rSunRot.*dSunRot(2,:);
    vSunRot = 1/2+rSunRot.*dSunRot(1,:);

    sunMask = fspecial('gaussian', size(sunPatch,1), size(sunPatch,1)/8);
    sunMask = (sunMask-min(sunMask(:))) ./ (max(sunMask(:))-min(sunMask(:)));
    sunMask(sunMask<0.01) = 0;

    %% Composite the sun into the environment map
    tSunPatch = zeros(size(angularEnvMap));
    for c=1:3
        tSunPatch(:,:,c) = griddata(reshape(uSunRot, size(sunPatch,1), size(sunPatch,2)), reshape(vSunRot, size(sunPatch,1), size(sunPatch,2)), ...
            sunPatch(:,:,c), reshape(uEnvMap, size(angularEnvMap,1), size(angularEnvMap,2)), reshape(vEnvMap, size(angularEnvMap,1), size(angularEnvMap,2)));
    end

    indValid = ~isnan(tSunPatch(:,:,1));
%     objectMask = zeros(size(tSunPatch(:,:,1)));
%     objectMask(indValid) = 1;
    tSunPatch(isnan(tSunPatch)) = 0;

    tMask = griddata(reshape(uSunRot, size(sunPatch,1), size(sunPatch,2)), reshape(vSunRot, size(sunPatch,1), size(sunPatch,2)), ...
        sunMask, reshape(uEnvMap, size(angularEnvMap,1), size(angularEnvMap,2)), reshape(vEnvMap, size(angularEnvMap,1), size(angularEnvMap,2)));
    tMask(~indValid) = 0;
    tMaskInv = ones(size(tMask)) - tMask;
    
    % make sure the sun isn't added below the horizon
    horizonMask = zeros(size(tSunPatch));
    horizonMask(1:size(horizonMask,1)/2,:,:) = 1;
    tSunPatch = tSunPatch.*envmapLatLong2Angular(horizonMask, size(tSunPatch,1));
    % pre-multiply by blending mask
    tSunPatch = tSunPatch.*repmat(tMask, [1 1 3]);
    
    % divide by integral 
%     tSunPatch = tSunPatch ./ repmat(max(max(tSunPatch, [], 1), [], 2), [size(tSunPatch, 1) size(tSunPatch, 2) 1]);
    tSunPatch = tSunPatch ./ repmat(sum(sum(tSunPatch, 1), 2), [size(tSunPatch, 1) size(tSunPatch, 2) 1]);
    
    % multiply by ratio of cur/max (zenith)
%     sunSumRatio = sunZenithBrightness.*(sunParams(1,:)+sunParams(2,:).*cos(sunZenith))./(sunParams(1,:)+sunParams(2,:));
%     sunSumRatio = sunZenithBrightness.*attOpticalMass(airMassModelParams(1,:), airMassModelParams(2,:), airMassModelParams(3,:), sunZenith) ./ attOpticalMass(airMassModelParams(1,:), airMassModelParams(2,:), airMassModelParams(3,:), 0);
    
    tSunPatch = tSunPatch .* repmat(permute(sunBrightness, [1 3 2]), [size(tSunPatch, 1) size(tSunPatch, 2) 1]);
    
    % integrate over sun patch
    sunColor = squeeze(sum(sum(tSunPatch, 1), 2));

    % compose the result
%     envMap = tSunPatch .* repmat(tMask, [1 1 3]) + angularEnvMap .* repmat(tMaskInv, [1 1 3]);
    envMap = tSunPatch + angularEnvMap .* repmat(tMaskInv, [1 1 3]);
    outputAngularEnvMap = envMap;
    
    fprintf('sun area: (%.2f, %.2f, %.2f)...', sunColor(1), sunColor(2), sunColor(3));
    envMapSum = squeeze(sum(sum(envMap,1),2));
    fprintf('env map area: (%.2f, %.2f, %.2f)\n', envMapSum(1), envMapSum(2), envMapSum(3));
%     fprintf('max sun patch: %.2f, max env map: %.2f\n', max(tSunPatch(:)), max(envMap(:)));

    %% Display
    if args.DoDisplay
        figure, imagesc(log(envMap(:,:,1))), colorbar;
    end
end