%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function envMap = envmapSequenceSky2EnvMap(im, varargin)
%   Builds the environment map for the sky of an image in a sequence.
% 
% Input parameters: 
%  - im: input image
%  - turbidity: 
%  - varargin: 
%    - DoDisplay: 1 or [0] displays partial results that illustrates the progression of the algorithm
%    - EnvMapHeight: [250] desired resulting environment map height
%    - Position: 1x3 vector indicating the 3-D position of the desired environment map
% 
% Output parameters: 
%  - envMap: the estimated environment map (Nx(N*2)), in lat-long format
%  - envMapMask: the ground mask projected onto the environment map
%  
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [envMap, indSky] = envmapSequenceSky2EnvMap(im, turbidity, scaleFactors, ...
    camAzimuth, sunZenith, sunAzimuth, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2006-2011 Jean-Francois Lalonde
% Carnegie Mellon University
% Do not distribute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Parse inputs
defaultArgs = struct('EnvMapHeight', 250, 'DoDisplay', 0, 'Position', [0 1 7.5]);
args = parseargs(defaultArgs, varargin{:});

nbRowsEnvMap = args.EnvMapHeight; % number of rows in output map
nbColsEnvMap = nbRowsEnvMap*2; % number of cols in output map

im = im2double(im);
[nr,nc,d] = size(im); %#ok

% Environment map center
centerPoint = args.Position(:);

envMap = zeros(nbRowsEnvMap, nbColsEnvMap, 3);

%% Get angles of sky pixels in the resulting environment map
[u,v] = meshgrid(0:2/(nbColsEnvMap-1):2, 0:1/(nbRowsEnvMap-1):1);

thetaLatLong = pi.*(u-1);
phiLatLong = pi.*v;

dxLatLong = sin(phiLatLong).*sin(thetaLatLong);
dyLatLong = cos(phiLatLong);
dzLatLong = -sin(phiLatLong).*cos(thetaLatLong);
dLatLong = [dxLatLong(:)'; dyLatLong(:)'; dzLatLong(:)'];

% rotate by -180deg about the y axis
theta = pi;
R = [cos(theta) 0 sin(theta); 0 1 0; -sin(theta) 0 cos(theta)];
dLatLongRot = R*dLatLong;

% compute the scale factor
lambda = -repmat(centerPoint(2), 1, size(dLatLongRot, 2)) ./ dLatLongRot(2,:);

% keep only negative lambdas!
indSky = lambda<=0;

%% Synthesize sky luminance (in RGB)
% compute angles wrt zenith and sun 
skyZenith = phiLatLong(indSky);

deltaAzimuth = sunAzimuth - camAzimuth + thetaLatLong(indSky);
skySunAngle = acos(cos(sunZenith) .* cos(skyZenith) + sin(sunZenith) .* sin(skyZenith) .* cos(deltaAzimuth));

skyParams = zeros(6, 3);
for c=1:3
    % get coefficients corresponding to turbidity
    sParams = getTurbidityMapping(c)*[turbidity 1]';
    skyParams(1:5, c) = sParams; skyParams(6, c) = scaleFactors(c);

    % compute luminance
    envMapChannel = zeros(nbRowsEnvMap, nbColsEnvMap);
    envMapChannel(indSky) = scaleFactors(c) .* perezSkyModel(sParams(1), sParams(2), sParams(3), sParams(4), sParams(5), skyZenith, skySunAngle) ./ ...
        perezSkyModel(sParams(1), sParams(2), sParams(3), sParams(4), sParams(5), 0, sunZenith);
    
    % add to environment map
    envMap(:,:,c) = envMapChannel;
end

% convert from xyY to rgb, but don't clamp to the [0,1] interval, keep HDR!
envMap = xyY2rgb(envMap, 0);

if args.DoDisplay
    figure(1); hold off;
    subplot(3,2,[1 3 5]), imshow(envMap);
    subplot(3,2,2), imagesc(envMap(:,:,1)); colorbar;
    subplot(3,2,4), imagesc(envMap(:,:,2)); colorbar;
    subplot(3,2,6), imagesc(envMap(:,:,3)); colorbar;
end

