%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function envmapImageCircle2EnvMap(im, varargin)
%  Extracts an environment map from a single image by taking a circle and projecting it onto a
%  sphere. (from Khan, Reinhard, et al. paper "Image-based material editing", SIGGRAPH 2006)
%
% Input parameters:
%  - im: background image name
%  - varargin: 
%    - EnvMapHeight: height (in pixels) of the environment map [default=250]
%
% Output parameters:
%  - envMap: corresponding environment map, in lat-long format
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function envMap = imageCircle2EnvMap(im, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2006-2011 Jean-Francois Lalonde
% Carnegie Mellon University
% Do not distribute
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Setup
% Parse inputs
defaultArgs = struct('EnvMapHeight', 250);
args = parseargs(defaultArgs, varargin{:});

nbRowsEnvMap = args.EnvMapHeight; % number of rows in output map
nbColsEnvMap = nbRowsEnvMap*2; % number of cols in output map

[nr,nc,d] = size(im); %#ok

%% Compute needed 3-D directions
[uLatLong, vLatLong] = meshgrid(0:2/(nbColsEnvMap-1):2, 0:1/(nbRowsEnvMap-1):1);

thetaLatLong = pi.*(uLatLong-1);
phiLatLong = pi.*vLatLong;

dxLatLong = sin(phiLatLong).*sin(thetaLatLong);
dyLatLong = cos(phiLatLong);

%% Find correspondences in the image

% Get the pixel normalized coordinates
xnImg = dxLatLong;
ynImg = -dyLatLong;

% Convert to image coordinates
radius = min(nr,nc) ./ 2;
xImg = xnImg .* radius + nc/2;
yImg = ynImg .* radius + nr/2;

%% Interpolate to find the intensity values
envMap = zeros(nbRowsEnvMap, nbColsEnvMap, 3);
for c=1:3
    envMap(:,:,c) = reshape(interp2(1:nc, 1:nr, im(:,:,c), min(max(xImg,1),nc), min(max(yImg,1),nr)), nbRowsEnvMap, nbColsEnvMap);
end
